@@ -3,17 +3,15 @@
 It also contains functions that are used by readers and utils. And it has the file-
 version definitions.
 """
-
 import datetime
+import importlib
 import logging
 import os
-import collections
+import pathlib
+import pickle
 import sys
 import time
-import importlib
-import pickle
-import warnings
-from functools import wraps
+from typing import Any, Tuple, Dict
 
 import numpy as np
 import pandas as pd
@@ -23,11 +21,11 @@ from cellpy.exceptions import NullData
 from cellpy.parameters import prms
 from cellpy.parameters.internal_settings import (
     ATTRS_CELLPYFILE,
-    cellpy_limits,
-    cellpy_units,
-    get_headers_summary,
     get_headers_normal,
     get_headers_step_table,
+    get_headers_summary,
+    get_default_raw_units,
+    get_default_raw_limits,
 )
 
 CELLPY_FILE_VERSION = 6
@@ -62,25 +60,25 @@ def pickle_protocol(level):
     return PickleProtocol(level)
 
 
-class FileID(object):
+class FileID:
     """class for storing information about the raw-data files.
 
-        This class is used for storing and handling raw-data file information.
-        It is important to keep track of when the data was extracted from the
-        raw-data files so that it is easy to know if the hdf5-files used for
-        @storing "treated" data is up-to-date.
+    This class is used for storing and handling raw-data file information.
+    It is important to keep track of when the data was extracted from the
+    raw-data files so that it is easy to know if the hdf5-files used for
+    @storing "treated" data is up-to-date.
 
-        Attributes:
-            name (str): Filename of the raw-data file.
-            full_name (str): Filename including path of the raw-data file.
-            size (float): Size of the raw-data file.
-            last_modified (datetime): Last time of modification of the raw-data
-                file.
-            last_accessed (datetime): last time of access of the raw-data file.
-            last_info_changed (datetime): st_ctime of the raw-data file.
-            location (str): Location of the raw-data file.
+    Attributes:
+        name (str): Filename of the raw-data file.
+        full_name (str): Filename including path of the raw-data file.
+        size (float): Size of the raw-data file.
+        last_modified (datetime): Last time of modification of the raw-data
+            file.
+        last_accessed (datetime): last time of access of the raw-data file.
+        last_info_changed (datetime): st_ctime of the raw-data file.
+        location (str): Location of the raw-data file.
 
-        """
+    """
 
     def __init__(self, filename=None):
         make_defaults = True
@@ -172,7 +170,7 @@ class FileID(object):
         return self.last_modified
 
 
-class Cell(object):
+class Cell:
     """Object to store data for a test.
 
     This class is used for storing all the relevant data for a 'run', i.e. all
@@ -233,16 +231,16 @@ class Cell(object):
 
         # meta-data
         self.cell_no = None
-        self.mass = prms.Materials["default_mass"]  # active material (in mg)
-        self.tot_mass = prms.Materials["default_mass"]  # total material (in mg)
+        self.mass = prms.Materials.default_mass  # active material (in mg)
+        self.tot_mass = prms.Materials.default_mass  # total material (in mg)
         self.no_cycles = 0.0
         self.charge_steps = None
         self.discharge_steps = None
         self.ir_steps = None
         self.ocv_steps = None
-        self.nom_cap = prms.DataSet["nom_cap"]  # mAh/g (for finding c-rates)
+        self.nom_cap = prms.DataSet.nom_cap  # mAh/g (for finding c-rates)
         self.mass_given = False
-        self.material = prms.Materials["default_material"]
+        self.material = prms.Materials.default_material
         self.merged = False
         self.file_errors = None  # not in use at the moment
         self.loaded_from = None  # loaded from (can be list if merged)
@@ -256,6 +254,8 @@ class Cell(object):
         self.name = None
 
         # new meta data
+        self.voltage_lim_low = None
+        self.voltage_lim_high = None
         self.cycle_mode = prms.Reader.cycle_mode
         self.active_electrode_area = None  # [cm2]
         self.active_electrode_thickness = None  # [micron]
@@ -285,18 +285,12 @@ class Cell(object):
 
         self.raw_data_files = []
         self.raw_data_files_length = []
-        self.raw_units = cellpy_units
-        self.raw_limits = cellpy_limits
-
-        # self.data = collections.OrderedDict()  # not used
-        # self.summary = collections.OrderedDict()  # not used
+        self.raw_units = get_default_raw_units()
+        self.raw_limits = get_default_raw_limits()
 
         self.raw = pd.DataFrame()
         self.summary = pd.DataFrame()
-        # self.summary_made = False  # Should be removed
-        self.steps = collections.OrderedDict()  # is this used? - check!
-        # self.step_table_made = False  # Should be removed
-        # self.parameter_table = collections.OrderedDict()
+        self.steps = pd.DataFrame()  # is this used? - check!
         self.summary_table_version = SUMMARY_TABLE_VERSION
         self.step_table_version = STEP_TABLE_VERSION
         self.cellpy_file_version = CELLPY_FILE_VERSION
@@ -370,7 +364,7 @@ class Cell(object):
         return txt
 
     @property
-    def summary_made(self):
+    def has_summary(self):
         """check if the summary table exists"""
         try:
             empty = self.summary.empty
@@ -379,7 +373,7 @@ class Cell(object):
         return not empty
 
     @property
-    def steps_made(self):
+    def has_steps(self):
         """check if the step table exists"""
         try:
             empty = self.steps.empty
@@ -388,14 +382,134 @@ class Cell(object):
         return not empty
 
     @property
-    def no_data(self):
-        # TODO: @jepe should consider renaming this to be in-line with "steps_made" etc. (or renaming steps_made and
-        #   summary_made to e.g. no_steps, no_summary)
+    def has_data(self):
         try:
             empty = self.raw.empty
         except AttributeError:
             empty = True
-        return empty
+        return not empty
+
+
+class InstrumentFactory:
+    def __init__(self):
+        self._builders = {}
+        self._kwargs = {}
+
+    def register_builder(self, key: str, builder: Tuple[str, Any], **kwargs) -> None:
+        """register an instrument loader module.
+
+        Args:
+            key: instrument id
+            builder: (module_name, module_path)
+            **kwargs: stored in the factory (will be used in the future for allowing to set
+               defaults to the builders to allow for using .query).
+        """
+
+        logging.debug(f"Registering instrument {key}")
+        self._builders[key] = builder
+        self._kwargs[key] = kwargs
+
+    def create(self, key: str, **kwargs):
+        """Create the instrument loader module and initialize the loader class.
+
+        Args:
+            key: instrument id
+            **kwargs: sent to the initializer of the loader class.
+
+        Returns:
+            instance of loader class.
+        """
+
+        module_name, module_path = self._builders.get(key, (None, None))
+
+        # constant:
+        instrument_class = "DataLoader"
+
+        if not module_name:
+            raise ValueError(key)
+
+        spec = importlib.util.spec_from_file_location(module_name, module_path)
+        loader_module = importlib.util.module_from_spec(spec)
+        sys.modules[module_name] = loader_module
+        spec.loader.exec_module(loader_module)
+        cls = getattr(loader_module, instrument_class)
+
+        # TODO: get stored kwargs from self.__kwargs and merge them with the supplied kwargs
+        #  (supplied should have preference)
+
+        return cls(**kwargs)
+
+    def query(self, key: str, variable: str) -> Any:
+        """performs a get_params lookup for the instrument loader.
+
+        Args:
+            key: instrument id.
+            variable: the variable you want to lookup.
+
+        Returns:
+            The value of the variable if the loaders get_params method supports it.
+        """
+        loader = self.create(key)
+        try:
+            value = loader.get_params(variable)
+            logging.debug(f"GOT {variable}={value} for {key}")
+            return value
+
+        except (AttributeError, NotImplementedError, KeyError):
+            logging.debug(f"COULD NOT RETRIEVE {variable} for {key}")
+        return
+
+
+def generate_default_factory():
+    instrument_factory = InstrumentFactory()
+    instruments = find_all_instruments()
+    for instrument_id, instrument in instruments.items():
+        instrument_factory.register_builder(instrument_id, instrument)
+    return instrument_factory
+
+
+def find_all_instruments() -> Dict[str, Tuple[str, str]]:
+    """finds all the supported instruments"""
+
+    import cellpy.readers.instruments as hard_coded_instruments_site
+
+    instruments_found = {}
+    logging.debug("Searching for modules in base instrument folder:")
+
+    hard_coded_instruments_site = pathlib.Path(
+        hard_coded_instruments_site.__file__
+    ).parent
+    modules_in_hard_coded_instruments_site = [
+        s
+        for s in hard_coded_instruments_site.glob("*.py")
+        if not (
+            str(s.name).startswith("_")
+            or str(s.name).startswith("dev_")
+            or str(s.name).startswith("base")
+            or str(s.name).startswith("backup")
+            or str(s.name).startswith("registered_loaders")
+        )
+    ]
+
+    for module_path in modules_in_hard_coded_instruments_site:
+        module_name = module_path.name.rstrip(".py")
+        logging.debug(module_name)
+        instruments_found[module_name] = (
+            module_name,
+            module_path,
+        )
+        logging.debug(" -> added")
+
+    logging.debug("Searching for module configurations in user instrument folder:")
+    # These are only yaml-files and should ideally import the appropriate
+    #    custom loader class
+    # Might not be needed.
+    logging.debug("- Not implemented yet")
+
+    logging.debug("Searching for modules through plug-ins:")
+    # Not sure how to do this yet. Probably also some importlib trick.
+    logging.debug("- Not implemented yet")
+    return instruments_found
 
 
 def identify_last_data_point(data):
@@ -600,23 +714,23 @@ def interpolate_y_on_x(
 ):
     """Interpolate a column based on another column.
 
-        Args:
-            df: DataFrame with the (cycle) data.
-            x: Column name for the x-value (defaults to the step-time column).
-            y: Column name for the y-value (defaults to the voltage column).
-            new_x (numpy array or None): Interpolate using these new x-values
-                instead of generating x-values based on dx or number_of_points.
-            dx: step-value (defaults to 10.0)
-            number_of_points: number of points for interpolated values (use
-                instead of dx and overrides dx if given).
-            direction (-1,1): if direction is negative, then invert the
-                x-values before interpolating.
-            **kwargs: arguments passed to scipy.interpolate.interp1d
+    Args:
+        df: DataFrame with the (cycle) data.
+        x: Column name for the x-value (defaults to the step-time column).
+        y: Column name for the y-value (defaults to the voltage column).
+        new_x (numpy array or None): Interpolate using these new x-values
+            instead of generating x-values based on dx or number_of_points.
+        dx: step-value (defaults to 10.0)
+        number_of_points: number of points for interpolated values (use
+            instead of dx and overrides dx if given).
+        direction (-1,1): if direction is negative, then invert the
+            x-values before interpolating.
+        **kwargs: arguments passed to scipy.interpolate.interp1d
 
-        Returns: DataFrame with interpolated y-values based on given or
-            generated x-values.
+    Returns: DataFrame with interpolated y-values based on given or
+        generated x-values.
 
-        """
+    """
 
     # TODO: allow for giving a fixed interpolation range (x-values).
     #  Remember to treat extrapolation properly (e.g. replace with NaN?).
