@@ -1,57 +1,43 @@
-import time
+import logging
 import os
 import pathlib
+import re
 import tempfile
+import time
 import warnings
-import logging
+from dataclasses import asdict
+from datetime import datetime
 
-import pandas as pd
 import numpy as np
+import pandas as pd
 
 from cellpy.parameters import prms
 
-
 # logger = logging.getLogger(__name__)
 
-# Note 14.12.2020: xlrd has explicitly removed support for anything other than xls files
-# Solution: install openpyxl
-# df1=pd.read_excel(
-#      os.path.join(APP_PATH, "Data", "aug_latest.xlsm"),
-#      sheet_name=None,
-#      engine='openpyxl',
-# )
 
-
-class DbSheetCols(object):
+class DbSheetCols:
     def __init__(self, level=0):
-        for table_key in prms.DbCols:
-            setattr(self, table_key, prms.DbCols[table_key][level])
+        db_cols_from_prms = asdict(prms.DbCols)
+        for table_key, values in db_cols_from_prms.items():
+            setattr(self, table_key, values[level])
 
     def __repr__(self):
         return f"<DbCols: {self.__dict__}>"
 
 
-class Reader(object):
-    def __init__(self, db_file=None, db_datadir=None, db_datadir_processed=None):
+class Reader:
+    def __init__(
+        self, db_file=None, db_datadir=None, db_datadir_processed=None, db_frame=None
+    ):
+        """Simple excel reader.
 
-        if not db_file:
-            self.db_path = prms.Paths["db_path"]
-            self.db_filename = prms.Paths["db_filename"]
-            self.db_file = os.path.join(self.db_path, self.db_filename)
-        else:
-            self.db_path = os.path.dirname(db_file)
-            self.db_filename = os.path.basename(db_file)
-            self.db_file = db_file
-
-        if not db_datadir:
-            self.db_datadir = prms.Paths["rawdatadir"]
-        else:
-            self.db_datadir = db_datadir
-
-        if not db_datadir_processed:
-            self.db_datadir_processed = prms.Paths["cellpydatadir"]
-        else:
-            self.db_datadir_processed = db_datadir_processed
+        Args:
+            db_file (str, pathlib.Path): xlsx-file to read.
+            db_datadir(str, pathlib.Path): path where raw date is located.
+            db_datadir_processed (str, pathlib.Path): path where cellpy files are located.
+            db_frame (pandas.DataFrame): use this instead of reading from xlsx-file.
+        """
 
         self.db_sheet_table = prms.Db.db_table_name
         self.db_header_row = prms.Db.db_header_row
@@ -63,11 +49,38 @@ class Reader(object):
         self.db_sheet_cols = DbSheetCols()
         self.db_sheet_cols_units = DbSheetCols(1)
 
-        self.skiprows, self.nrows = self._find_out_what_rows_to_skip()
+        if not db_datadir:
+            self.db_datadir = prms.Paths.rawdatadir
+        else:
+            self.db_datadir = db_datadir
+
+        if not db_datadir_processed:
+            self.db_datadir_processed = prms.Paths.cellpydatadir
+        else:
+            self.db_datadir_processed = db_datadir_processed
+
+        if not db_file:
+            self.db_path = prms.Paths.db_path
+            self.db_filename = prms.Paths.db_filename
+            self.db_file = os.path.join(self.db_path, self.db_filename)
+        else:
+            self.db_path = os.path.dirname(db_file)
+            self.db_filename = os.path.basename(db_file)
+            self.db_file = db_file
+
         self.dtypes_dict = self._create_dtypes_dict()
         self.headers = self.dtypes_dict.keys()
-        logging.debug("opening sheet")
-        self.table = self._open_sheet()
+
+        if db_frame is not None:
+            print("Using frame instead of file")
+            self.table = db_frame.copy()
+        else:
+
+            self.skiprows, self.nrows = self._find_out_what_rows_to_skip()
+            logging.debug("opening sheet")
+
+            self.table = self._open_sheet()
+
         logging.debug("got table")
         logging.debug(self.table)
 
@@ -78,6 +91,84 @@ class Reader(object):
         txt += str(self.table.head())
         return txt
 
+    @staticmethod
+    def _parse_argument_str(argument_str: str) -> dict:
+        # the argument str must be on the form:
+        # "keyword-1=value-1;keyword-2=value2"
+        if argument_str is None:
+            return
+        sep = ";"
+        parts = [part.strip() for part in argument_str.split(sep=sep)]
+        sep = "="
+        arguments = {}
+        for p in parts:
+            k, v = p.split(sep=sep)
+            arguments[k.strip()] = v.strip()
+        return arguments
+
+    @staticmethod
+    def _extract_date_from_cell_name(
+        cell_name, strf="%Y%m%d", regexp=None, splitter="_", position=0, start=0, end=12
+    ):
+        """Extract date given a cell name (or filename).
+
+        Uses regexp if given to find date txt, if not it uses splitter if splitter is not None or "",
+        else start-stop. Uses strf to parse for date in date txt.
+        if regexp is "auto", regexp is interpreted from strf
+
+        Args:
+            cell_name (str): extract date from.
+            strf (str): datetime string formatter.
+            regexp (str | "auto"): regular expression.
+            splitter (str): split parts into sub-parts.
+            position (int): selected sub-part.
+            start (int): number of first character in the date part.
+            end (int): number of last character in the date part.
+
+        Returns:
+            datetime.datetime object
+        """
+
+        if regexp is not None:
+            if regexp == "auto":
+                year_r = r"%Y"
+                month_r = r"%m"
+                day_r = r"%d"
+
+                regexp = strf.replace("\\", "\\\\").replace("-", r"\-")
+                regexp = (
+                    regexp.replace(year_r, "[0-9]{4}")
+                    .replace(month_r, "[0-9]{2}")
+                    .replace(day_r, "[0-9]{2}")
+                )
+                regexp = f"({regexp})"
+
+            m = re.search(regexp, cell_name)
+            datestr = m[0]
+
+        elif splitter:
+            datestr = cell_name.split(splitter)[position]
+
+        else:
+            datestr = cell_name[start:end]
+
+        try:
+
+            date = datetime.strptime(datestr, strf)
+        except ValueError as e:
+            logging.debug(e)
+            return None
+
+        return date
+
+    def extract_date_from_cell_name(self, force=False):
+        if force or "date" not in self.table.columns:
+            self.table = self.table.assign(
+                date=self.table.file_name_indicator.apply(
+                    self._extract_date_from_cell_name
+                )
+            )
+
     # --------not fixed from here -------------------------------
 
     def _find_out_what_rows_to_skip(self):
@@ -107,9 +198,9 @@ class Reader(object):
         units = {
             "int": np.int32,
             "float": np.float64,
-            "str": np.str,
-            "bol": np.bool,
-            "cat": np.str,
+            "str": str,
+            "bol": bool,
+            "cat": str,
         }
         return units.get(label.lower(), object)
 
@@ -132,6 +223,15 @@ class Reader(object):
 
     def _open_sheet(self, dtypes_dict=None):
         """Opens sheets and returns it"""
+
+        # Note 14.12.2020: xlrd has explicitly removed support for anything other than xls files
+        # Solution: install openpyxl
+        # df1=pd.read_excel(
+        #      os.path.join(APP_PATH, "Data", "aug_latest.xlsm"),
+        #      sheet_name=None,
+        #      engine='openpyxl',
+        # )
+
         table_name = self.db_sheet_table
         header_row = self.db_header_row
         nrows = self.nrows
@@ -144,7 +244,7 @@ class Reader(object):
         logging.debug(f"Number of rows (no means all): {nrows}")
         logging.debug(f"Skipping the following rows: {rows_to_skip}")
         logging.debug(f"Declaring the following dtyps: {dtypes_dict}")
-        work_book = pd.ExcelFile(self.db_file, engine='openpyxl')
+        work_book = pd.ExcelFile(self.db_file, engine="openpyxl")
         try:
             sheet = work_book.parse(
                 table_name,
@@ -170,7 +270,7 @@ class Reader(object):
 
         Returns:
             True if OK, False if not.
-            """
+        """
         probably_good_to_go = True
         sheet = self.table
         identity = self.db_sheet_cols.id
@@ -287,6 +387,10 @@ class Reader(object):
         insp = self._pick_info(serial_number, column_name)
         return insp
 
+    def get_by_column_label(self, column_name, serial_number):
+        insp = self._pick_info(serial_number, column_name)
+        return insp
+
     def get_group(self, serial_number):
         column_name = self.db_sheet_cols.group
         insp = self._pick_info(serial_number, column_name)
@@ -302,9 +406,6 @@ class Reader(object):
                 "Could not read the cycle mode (using value from prms instead)"
             )
             logging.debug(f"cycle mode: {prms.Reader.cycle_mode}")
-            import sys
-
-            sys.exit()
             return prms.Reader.cycle_mode
 
     def get_loading(self, serial_number):
@@ -315,6 +416,19 @@ class Reader(object):
     def get_areal_loading(self, serial_number):
         raise NotImplementedError
 
+    def get_args(self, serial_number: int) -> dict:
+        column_name = self.db_sheet_cols.argument
+        argument_str = self._pick_info(serial_number, column_name)
+        try:
+            argument = self._parse_argument_str(argument_str)
+        except Exception as e:
+            logging.warning(f"could not parse argument str:")
+            logging.warning(f"{argument_str}")
+            logging.warning(f"Error message: {e}")
+            return {}
+
+        return argument
+
     def get_mass(self, serial_number):
         column_name_mass = self.db_sheet_cols.active_material
         mass = self._pick_info(serial_number, column_name_mass)
@@ -328,6 +442,10 @@ class Reader(object):
         column_name = self.db_sheet_cols.experiment_type
         return self._pick_info(serial_number, column_name)
 
+    def get_instrument(self, serial_number):
+        column_name = self.db_sheet_cols.instrument
+        return self._pick_info(serial_number, column_name)
+
     def get_total_mass(self, serial_number):
         column_name_mass = self.db_sheet_cols.total_material
         total_mass = self._pick_info(serial_number, column_name_mass)
@@ -336,13 +454,13 @@ class Reader(object):
     def get_all(self):
         return self.filter_by_col([self.db_sheet_cols.id, self.db_sheet_cols.exists])
 
-    def get_fileid(self, serialno, full_path=True):
+    def get_fileid(self, serial_number, full_path=True):
         column_name = self.db_sheet_cols.file_name_indicator
         if not full_path:
-            filename = self._pick_info(serialno, column_name)
+            filename = self._pick_info(serial_number, column_name)
         else:
             filename = os.path.join(
-                self.db_datadir_processed, self._pick_info(serialno, column_name)
+                self.db_datadir_processed, self._pick_info(serial_number, column_name)
             )
         return filename
 
@@ -495,7 +613,7 @@ class Reader(object):
 
         return sheet.loc[:, identity].values.astype(int)
 
-    def select_batch(self, batch, batch_col_name=None):
+    def select_batch(self, batch, batch_col_name=None, case_sensitive=True):
         """selects the rows  in column batch_col_number
         (default: DbSheetCols.batch)"""
 
@@ -506,7 +624,11 @@ class Reader(object):
         identity = self.db_sheet_cols.id
         exists_col_number = self.db_sheet_cols.exists
 
-        criterion = sheet.loc[:, batch_col_name] == batch
+        if case_sensitive:
+            criterion = sheet.loc[:, batch_col_name] == batch
+        else:
+            criterion = (sheet.loc[:, batch_col_name]).upper() == batch.upper()
+
         exists = sheet.loc[:, exists_col_number] > 0
         # This will crash if the col is not of dtype number
         sheet = sheet[criterion & exists]
