@@ -1,39 +1,42 @@
 """Routines for batch processing of cells (v2)."""
 
 import logging
+import os
 import pathlib
 import shutil
-import warnings
-import os
 import sys
+import warnings
 
 import pandas as pd
+from pandas import Index
 from tqdm.auto import tqdm
 
-from cellpy import prms
-from cellpy import log
 import cellpy.exceptions
+from cellpy import log, prms
 from cellpy.parameters.internal_settings import (
-    get_headers_step_table,
-    get_headers_journal,
-    get_headers_summary,
+    headers_journal,
+    headers_step_table,
+    headers_summary,
 )
-from cellpy.utils.batch_tools.batch_exporters import CSVExporter
+from cellpy.utils.batch_tools.batch_analyzers import (
+    BaseSummaryAnalyzer,
+    OCVRelaxationAnalyzer,
+)
+from cellpy.utils.batch_tools.batch_core import Data
 from cellpy.utils.batch_tools.batch_experiments import CyclingExperiment
-from cellpy.utils.batch_tools.batch_plotters import CyclingSummaryPlotter
-from cellpy.utils.batch_tools.batch_analyzers import OCVRelaxationAnalyzer, BaseSummaryAnalyzer
+from cellpy.utils.batch_tools.batch_exporters import CSVExporter
 from cellpy.utils.batch_tools.batch_journals import LabJournal
+from cellpy.utils.batch_tools.batch_plotters import CyclingSummaryPlotter
 from cellpy.utils.batch_tools.dumpers import ram_dumper
 
 # logger = logging.getLogger(__name__)
 logging.captureWarnings(True)
-hdr_journal = get_headers_journal()
-hdr_summary = get_headers_summary()
 
 COLUMNS_SELECTED_FOR_VIEW = [
-    hdr_journal.mass,
-    # hdr_journal.total_mass,
-    hdr_journal.loading,
+    headers_journal.mass,
+    headers_journal.total_mass,
+    headers_journal.loading,
+    headers_journal.nom_cap,
 ]
 
 
@@ -59,6 +62,7 @@ class Batch:
             file_name (str or pathlib.Path): journal file name to load.
             db_reader (str): data-base reader to use (defaults to "default" as given
                 in the config-file or prm-class).
+            frame (pandas.DataFrame): load from given dataframe.
         Args:
             *args: name (str) (project (str))
 
@@ -75,6 +79,8 @@ class Batch:
             nom_cap (float): give a nominal capacity if you want to use another value than
                 the one given in the config-file or prm-class.
         """
+        # TODO: add option for setting max cycle number
+        #   use self.experiment.last_cycle = xxx
         default_log_level = kwargs.pop("log_level", None)
         custom_log_dir = kwargs.pop("custom_log_dir", None)
         if default_log_level is not None or custom_log_dir is not None:
@@ -87,6 +93,7 @@ class Batch:
         db_reader = kwargs.pop("db_reader", "default")
 
         file_name = kwargs.pop("file_name", None)
+        frame = kwargs.pop("frame", None)
 
         logging.debug("creating CyclingExperiment")
         self.experiment = CyclingExperiment(db_reader=db_reader)
@@ -102,22 +109,24 @@ class Batch:
         self.experiment.nom_cap = kwargs.pop("nom_cap", None)
 
         if not file_name:
-            if len(args) > 0:
-                self.experiment.journal.name = args[0]
+            if frame is not None:
+                self.experiment.journal.from_frame(frame, **kwargs)
+            else:
+                if len(args) > 0:
+                    self.experiment.journal.name = args[0]
 
-            if len(args) > 1:
-                self.experiment.journal.project = args[1]
-
-            for key in kwargs:
-                if key == "name":
-                    self.experiment.journal.name = kwargs[key]
-                elif key == "project":
-                    self.experiment.journal.project = kwargs[key]
-                elif key == "batch_col":
-                    self.experiment.journal.batch_col = kwargs[key]
+                if len(args) > 1:
+                    self.experiment.journal.project = args[1]
 
+                for key in kwargs:
+                    if key == "name":
+                        self.experiment.journal.name = kwargs[key]
+                    elif key == "project":
+                        self.experiment.journal.project = kwargs[key]
+                    elif key == "batch_col":
+                        self.experiment.journal.batch_col = kwargs[key]
         else:
-            self.experiment.journal.from_file(file_name=file_name)
+            self.experiment.journal.from_file(file_name=file_name, **kwargs)
 
         self.exporter = CSVExporter()
         self.exporter._assign_dumper(ram_dumper)
@@ -129,7 +138,7 @@ class Batch:
         self.plotter = CyclingSummaryPlotter()
         self.plotter.assign(self.experiment)
         self._journal_name = self.journal_name
-        self.headers_step_table = get_headers_step_table()
+        self.headers_step_table = headers_step_table
 
     def __str__(self):
         return str(self.experiment)
@@ -151,6 +160,10 @@ class Batch:
         pages = pages[COLUMNS_SELECTED_FOR_VIEW]
         return pages
 
+    @property
+    def name(self):
+        return self.experiment.journal.name
+
     def _check_cell_raw(self, cell_id):
         try:
             c = self.experiment.data[cell_id]
@@ -176,7 +189,7 @@ class Batch:
         try:
             c = self.experiment.data[cell_id]
             s = c.cell.summary
-            return s[hdr_summary.charge_capacity].max()
+            return s[headers_summary.charge_capacity].max()
 
         except Exception as e:
             return None
@@ -185,7 +198,7 @@ class Batch:
         try:
             c = self.experiment.data[cell_id]
             s = c.cell.summary
-            return s[hdr_summary.charge_capacity].min()
+            return s[headers_summary.charge_capacity].min()
 
         except Exception as e:
             return None
@@ -194,7 +207,7 @@ class Batch:
         try:
             c = self.experiment.data[cell_id]
             s = c.cell.summary
-            return s[hdr_summary.charge_capacity].mean()
+            return s[headers_summary.charge_capacity].mean()
 
         except Exception as e:
             return None
@@ -203,7 +216,7 @@ class Batch:
         try:
             c = self.experiment.data[cell_id]
             s = c.cell.summary
-            return s[hdr_summary.charge_capacity].std()
+            return s[headers_summary.charge_capacity].std()
 
         except Exception as e:
             return None
@@ -225,24 +238,28 @@ class Batch:
     def drop(self, cell_label=None):
         """Drop cells from the journal.
 
-        If cell_label is not given, cellpy will look into the journal for session
+        If ``cell_label`` is not given, ``cellpy`` will look into the journal for session
         info about bad cells, and if it finds it, it will remove those from the
         journal.
 
         Note! remember to save your journal again after modifying it.
+
         Note! this method has not been properly tested yet.
 
         Args:
             cell_label (str): the cell label of the cell you would like to remove.
 
-        Returns: cellpy.utils.batch object (returns a copy if `keep_old` is True).
+        Returns:
+            ``cellpy.utils.batch`` object (returns a copy if `keep_old` is ``True``).
 
         """
         if cell_label is None:
             try:
                 cell_labels = self.journal.session["bad_cells"]
             except AttributeError:
-                logging.critical("session info about bad cells is missing - cannot drop")
+                logging.critical(
+                    "session info about bad cells is missing - cannot drop"
+                )
                 return
         else:
             cell_labels = [cell_label]
@@ -254,12 +271,12 @@ class Batch:
                 self.pages = self.pages.drop(cell_label)
 
     def report(self, stylize=True):
-        """ Create a report on all the cells in the batch object.
+        """Create a report on all the cells in the batch object.
 
         Remark! To perform a reporting, cellpy needs to access all the data (and it might take some time).
 
         Returns:
-            pandas.DataFrame
+            ``pandas.DataFrame``
         """
         pages = self.experiment.journal.pages
         pages = pages[COLUMNS_SELECTED_FOR_VIEW].copy()
@@ -332,13 +349,11 @@ class Batch:
 
     def _concat_memory_dumped(self, engine_name):
         keys = [df.name for df in self.experiment.memory_dumped[engine_name]]
-        return pd.concat(
-            self.experiment.memory_dumped[engine_name], keys=keys, axis=1
-        )
+        return pd.concat(self.experiment.memory_dumped[engine_name], keys=keys, axis=1)
 
     @property
     def summaries(self):
-        # should add link-mode?
+        """Concatenated summaries from all cells (multiindex dataframe)."""
         try:
             return self._concat_memory_dumped("summary_engine")
         except KeyError:
@@ -348,33 +363,59 @@ class Batch:
 
     @property
     def summary_headers(self):
+        """The column names of the concatenated summaries"""
         try:
             return self.summaries.columns.get_level_values(0)
         except AttributeError:
             logging.info("can't get any columns")
 
     @property
-    def cell_names(self):
+    def cell_names(self) -> list:
         return self.experiment.cell_names
 
     @property
-    def raw_headers(self):
-        return self.experiment.data[0].cell.raw.columns
+    def labels(self):
+        # Plan: allow cells to both have a label and a cell_name, where that latter should be a unique
+        # identifier. Consider also to allow for a group-name.
+        # The label and cell name can be the same. Consider allowing several cells to share the same label
+        # thus returning several cellpy cell objects. Our use "group" for this purpose.
+        print(
+            "Label-based look-up is not supported yet. Performing cell-name based look-up instead."
+        )
+        return self.experiment.cell_names
 
     @property
-    def step_headers(self):
-        return self.experiment.data[0].cell.steps.columns
+    def cells(self) -> Data:
+        """access cells as a Data object (attribute lookup and automatic loading)"""
+        return self.experiment.data
 
     @property
-    def pages(self):
+    def cell_summary_headers(self) -> Index:
+        return self.experiment.data[self.experiment.cell_names[0]].cell.summary.columns
+
+    @property
+    def cell_raw_headers(self) -> Index:
+        return self.experiment.data[self.experiment.cell_names[0]].cell.raw.columns
+
+    @property
+    def cell_step_headers(self) -> Index:
+        return self.experiment.data[self.experiment.cell_names[0]].cell.steps.columns
+
+    @property
+    def pages(self) -> pd.DataFrame:
         return self.experiment.journal.pages
 
     @pages.setter
-    def pages(self, df):
+    def pages(self, df: pd.DataFrame):
         self.experiment.journal.pages = df
+        all_cell_labels = set(self.experiment.cell_data_frames.keys())
+        cell_labels_to_keep = set(self.journal.pages.index)
+        cell_labels_to_remove = all_cell_labels - cell_labels_to_keep
+        for cell_label in cell_labels_to_remove:
+            del self.experiment.cell_data_frames[cell_label]
 
     @property
-    def journal(self):
+    def journal(self) -> LabJournal:
         return self.experiment.journal
 
     @journal.setter
@@ -386,7 +427,7 @@ class Batch:
             "the journal.pages instead."
         )
 
-    def create_journal(self, description=None, from_db=True):
+    def create_journal(self, description=None, from_db=True, **kwargs):
         """Create journal pages.
 
         This method is a wrapper for the different Journal methods for making
@@ -395,19 +436,21 @@ class Batch:
         using the methods available in Journal for now.
 
         Args:
-            description: the information and meta-data needed to generate the journal
-                pages.
-                "empty": create an empty journal
-                dictionary: create journal pages from a dictionary
-                pd.DataFrame: create  journal pages from a pandas DataFrame
-                filename.json: load cellpy batch file
-
-                filename.xlxs: create journal pages from an excel file
-                    (not implemented yet)
+            description: the information and meta-data needed to generate the journal pages ('empty': create an
+                empty journal; ``dict``: create journal pages from a dictionary; ``pd.DataFrame``: create journal pages
+                from a ``pandas.DataFrame``: 'filename.json': load cellpy batch file; 'filename.xlsx': create journal
+                pages from an Excel file).
             from_db (bool): Deprecation Warning: this parameter will be removed as it is
                 the default anyway. Generate the pages from a db (the default option).
                 This will be over-ridden if description is given.
+
+            **kwargs: sent to sub-function(s) (*e.g.* from_db -> simple_db_reader -> find_files ->
+                filefinder.search_for_files).
+
         """
+
+        # TODO (jepe): create option to update journal without looking for files
+
         logging.debug("Creating a journal")
         logging.debug(f"description: {description}")
         logging.debug(f"from_db: {from_db}")
@@ -418,7 +461,7 @@ class Batch:
             from_db = False
 
         if from_db:
-            self.experiment.journal.from_db()
+            self.experiment.journal.from_db(**kwargs)
             self.experiment.journal.to_file()
             self.duplicate_journal(prms.Paths.batchfiledir)
 
@@ -436,12 +479,8 @@ class Batch:
 
             if is_file:
                 logging.info(f"loading file {description}")
-                if description.suffix == ".json":
-                    logging.debug("loading batch json file")
+                if description.suffix in [".json", ".xlsx"]:
                     self.experiment.journal.from_file(description)
-                elif description.suffix == ".xlxs":
-                    logging.debug("loading excel file")
-                    warnings.warn("not implemented yet")
                 else:
                     warnings.warn("unknown file extension")
 
@@ -529,18 +568,18 @@ class Batch:
             self.experiment.journal.paginate()
             self.duplicate_journal(prms.Paths.batchfiledir)
 
-    def create_folder_structure(self):
+    def create_folder_structure(self) -> None:
         warnings.warn("Deprecated - use paginate instead.", DeprecationWarning)
         self.experiment.journal.paginate()
         logging.info("created folders")
 
-    def paginate(self):
+    def paginate(self) -> None:
         """Create the folders where cellpy will put its output."""
 
         self.experiment.journal.paginate()
         logging.info("created folders")
 
-    def save_journal(self):
+    def save_journal(self) -> None:
         """Save the journal (json-format).
 
         The journal file will be saved in the project directory and in the
@@ -549,11 +588,23 @@ class Batch:
         """
 
         # Remark! Got an recursive error when running on mac.
-        self.experiment.journal.to_file()
+        self.experiment.journal.to_file(to_project_folder=True, paginate=False)
+        logging.info("saved journal pages to project folder")
         self.duplicate_journal(prms.Paths.batchfiledir)
-        logging.info("saving journal pages")
+        logging.info("duplicated journal pages to batch dir")
+        self.duplicate_journal()
+        logging.info("duplicated journal pages to current dir")
 
-    def duplicate_journal(self, folder=None):
+    def export_journal(self, filename=None) -> None:
+        """Export the journal to xlsx."""
+        if filename is None:
+            filename = self.experiment.journal.file_name
+        filename = pathlib.Path(filename).with_suffix(".xlsx")
+        self.experiment.journal.to_file(
+            file_name=filename, to_project_folder=False, paginate=False
+        )
+
+    def duplicate_journal(self, folder=None) -> None:
         """Copy the journal to folder.
 
         Args:
@@ -574,7 +625,9 @@ class Batch:
         except shutil.SameFileError:
             logging.debug("same file exception encountered")
 
-    def duplicate_cellpy_files(self, location="standard"):
+    def duplicate_cellpy_files(
+        self, location: str = "standard", selector: dict = None, **kwargs
+    ) -> None:
         """Copy the cellpy files and make a journal with the new names available in
         the current folder.
 
@@ -586,10 +639,15 @@ class Batch:
                 "cellpydatadir": the stated cellpy data dir in your settings (prms)
             or if the location is not one of the above, use the actual value of the
                 location argument.
+            selector (dict): if given, the cellpy files are reloaded after duplicating and
+                modified based on the given selector(s).
+
+            kwargs: sent to update if selector is provided
 
         Returns:
             The updated journal pages.
         """
+
         pages = self.experiment.journal.pages
         cellpy_file_dir = pathlib.Path(prms.Paths.cellpydatadir)
 
@@ -622,54 +680,103 @@ class Batch:
                 shutil.copy(from_file, to_file)
             except shutil.SameFileError:
                 logging.info("Same file! No point in copying")
+            except FileNotFoundError:
+                logging.info("File not found! Cannot copy it!")
 
         # save the journal pages
         pages["cellpy_file_name"] = pages["new_cellpy_file_name"]
         self.experiment.journal.pages = pages[columns]
         journal_file_name = pathlib.Path(self.experiment.journal.file_name).name
-        logging.info(f"saving journal to {journal_file_name}")
-        self.experiment.journal.to_file(journal_file_name)
+        self.experiment.journal.to_file(
+            journal_file_name, paginate=False, to_project_folder=False
+        )
+        if selector is not None:
+            logging.info("Modifying the cellpy-files.")
+            logging.info(f"selector: {selector}")
+            self.experiment.force_cellpy = True
+            self.update(selector=selector, **kwargs)
 
-        # return pages
-
-    # TODO: load_journal
     # TODO: list_journals?
 
-    def link(self):
-        """Link journal content to the cellpy-files and load the step information."""
-        self.experiment.link()
+    def link(self, max_cycle=None, force_combine_summaries=False) -> None:
+        """Link journal content to the cellpy-files and load the step information.
 
-    def load(self):
+        Args:
+            max_cycle (int): set maximum cycle number to link to.
+            force_combine_summaries (bool): automatically run combine_summaries (set this to True
+                if you are re-linking without max_cycle for a batch that previously were linked
+                with max_cycle)
+
+        """
+
+        self.experiment.link(max_cycle=max_cycle)
+        if force_combine_summaries or max_cycle:
+            self.summary_collector.do(reset=True)
+
+    def load(self) -> None:
         # does the same as update
         warnings.warn("Deprecated - use update instead.", DeprecationWarning)
         self.experiment.update()
 
-    def update(self, **kwargs):
-        """Load cells as defined in the journal"""
+    def update(self, **kwargs) -> None:
+        """Updates the selected datasets.
+
+        Args:
+            all_in_memory (bool): store the `cellpydata` in memory (default
+                False)
+            cell_specs (dict of dicts): individual arguments pr. cell. The `cellspecs` key-word argument
+                dictionary will override the **kwargs and the parameters from the journal pages
+                for the indicated cell.
+
+        kwargs:
+            transferred all the way to the instrument loader, if not
+            picked up earlier. Remark that you can obtain the same pr. cell by
+            providing a `cellspecs` dictionary. The kwargs have precedence over the
+            parameters given in the journal pages, but will be overridden by parameters
+            given by `cellspecs`.
+
+            Merging:
+                recalc (Bool): set to False if you don't want automatic "recalc" of
+                    cycle numbers etc. when merging several data-sets.
+            Loading:
+                selector (dict): selector-based parameters sent to the cellpy-file loader (hdf5) if
+                loading from raw is not necessary (or turned off).
+
+        """
         self.experiment.errors["update"] = []
         self.experiment.update(**kwargs)
 
-    def recalc(self, **kwargs):
+    def export_cellpy_files(self, path=None, **kwargs) -> None:
+        if path is None:
+            path = pathlib.Path(".").resolve()
+        self.experiment.errors["export_cellpy_files"] = []
+        self.experiment.export_cellpy_files(path=path, **kwargs)
+
+    def recalc(self, **kwargs) -> None:
         self.experiment.errors["recalc"] = []
         self.experiment.recalc(**kwargs)
 
-    def make_summaries(self):
+    def make_summaries(self) -> None:
         warnings.warn("Deprecated - use combine_summaries instead.", DeprecationWarning)
+
         self.exporter.do()
 
-    def combine_summaries(self, export_to_csv=True):
+    def combine_summaries(self, export_to_csv=True, **kwargs) -> None:
         """Combine selected columns from each of the cells into single frames"""
+
         if export_to_csv:
             self.exporter.do()
         else:
-            self.summary_collector.do()
+            self.summary_collector.do(**kwargs)
 
-    def plot_summaries(self, output_filename=None, backend=None):
+    def plot_summaries(
+        self, output_filename=None, backend=None, reload_data=False, **kwargs
+    ) -> None:
         """Plot the summaries"""
 
-        if "summary_engine" not in self.experiment.memory_dumped:
+        if reload_data or ("summary_engine" not in self.experiment.memory_dumped):
             logging.debug("running summary_collector")
-            self.summary_collector.do()
+            self.summary_collector.do(reset=True)
 
         if backend is None:
             backend = prms.Batch.backend
@@ -695,49 +802,59 @@ class Batch:
                     "could not find the bokeh module -> using matplotlib instead"
                 )
 
-        self.plotter.do()
+        self.plotter.do(**kwargs)
 
 
-def init(*args, **kwargs):
+def init(*args, **kwargs) -> Batch:
     """Returns an initialized instance of the Batch class.
 
     Args:
         *args: passed directly to Batch()
-            name: name of batch
-            project: name of project
-            batch_col: batch column identifier
+            **name**: name of batch;
+            **project**: name of project;
+            **batch_col**: batch column identifier.
         **kwargs:
-            file_name: json file if loading from pages
-            default_log_level: "INFO" or "DEBUG"
-            The rest is passed directly to Batch()
+            **file_name**: json file if loading from pages;
+            **default_log_level**: "INFO" or "DEBUG";
+            the rest is passed directly to Batch().
 
-    Usage:
+    Examples:
         >>> empty_batch = Batch.init(db_reader=None)
         >>> batch_from_file = Batch.init(file_name="cellpy_batch_my_experiment.json")
         >>> normal_init_of_batch = Batch.init()
     """
+    # TODO: add option for setting max cycle number (experiment.last_cycle)
     # set up cellpy logger
     default_log_level = kwargs.pop("default_log_level", None)
+    testing = kwargs.pop("testing", None)
     file_name = kwargs.pop("file_name", None)
+    frame = kwargs.pop("frame", None)
 
-    log.setup_logging(default_level=default_log_level, reset_big_log=True)
+    log.setup_logging(
+        default_level=default_log_level, testing=testing, reset_big_log=True
+    )
 
     logging.debug(f"returning Batch(kwargs: {kwargs})")
     if file_name is not None:
         kwargs.pop("db_reader", None)
         return Batch(*args, file_name=file_name, db_reader=None, **kwargs)
+    if frame is not None:
+        kwargs.pop("db_reader", None)
+        return Batch(*args, file_name=None, db_reader=None, frame=frame, **kwargs)
+
     return Batch(*args, **kwargs)
 
 
-def from_journal(journal_file, autolink=True):
+def from_journal(journal_file, autolink=True) -> Batch:
     """Create a Batch from a journal file"""
+    # TODO: add option for setting max cycle number (experiment.last_cycle)
     b = init(db_reader=None, file_name=journal_file)
     if autolink:
         b.link()
     return b
 
 
-def load_pages(file_name):
+def load_pages(file_name) -> pd.DataFrame:
     """Retrieve pages from a Journal file.
 
     This function is here to let you easily inspect a Journal file without
@@ -752,26 +869,25 @@ def load_pages(file_name):
         pandas.DataFrame
 
     """
+
     logging.info(f"Loading pages from {file_name}")
     try:
         pages, *_ = LabJournal.read_journal_jason_file(file_name)
+        return pages
     except cellpy.exceptions.UnderDefined:
         logging.critical("could not find any pages.")
-        return
-
-    return pages
 
 
-def process_batch(*args, **kwargs):
+def process_batch(*args, **kwargs) -> Batch:
     """Execute a batch run, either from a given file_name or by giving the name and project as input.
 
-    Usage:
-        process_batch(file_name | (name, project), **kwargs)
+    Examples:
+        >>> process_batch(file_name | (name, project), **kwargs)
 
     Args:
         *args: file_name or name and project (both string)
 
-    Optional keyword arguments:
+    Keyword Args:
         backend (str): what backend to use when plotting ('bokeh' or 'matplotlib').
             Defaults to 'matplotlib'.
         dpi (int): resolution used when saving matplotlib plot(s). Defaults to 300 dpi.
@@ -779,7 +895,7 @@ def process_batch(*args, **kwargs):
             'CRITICAL', 'DEBUG', or 'INFO'. The default is 'CRITICAL' (i.e. usually no log output to console).
 
     Returns:
-        cellpy.batch.Batch object
+        ``cellpy.batch.Batch`` object
     """
     silent = kwargs.pop("silent", False)
     backend = kwargs.pop("backend", None)
@@ -860,7 +976,7 @@ def iterate_batches(folder, extension=".json", glob_pattern=None, **kwargs):
         folder (str or pathlib.Path): folder containing the journal files.
         extension (str): extension for the journal files (used when creating a default glob-pattern).
         glob_pattern (str): optional glob pattern.
-        **kwargs: keyword arguments passed to ´batch.process_batch´.
+        **kwargs: keyword arguments passed to ``batch.process_batch``.
     """
 
     folder = pathlib.Path(folder)
@@ -916,7 +1032,7 @@ def iterate_batches(folder, extension=".json", glob_pattern=None, **kwargs):
     print("\n...Finished ")
 
 
-def main():
+def check_standard():
     from pathlib import Path
 
     # Use these when working on my work PC:
@@ -931,12 +1047,12 @@ def main():
     out_data_path = Path(out_data_path)
 
     logging.info("---SETTING SOME PRMS---")
-    prms.Paths["db_filename"] = "cellpy_db.xlsx"
-    prms.Paths["cellpydatadir"] = test_data_path / "hdf5"
-    prms.Paths["outdatadir"] = out_data_path
-    prms.Paths["rawdatadir"] = test_data_path / "data"
-    prms.Paths["db_path"] = test_data_path / "db"
-    prms.Paths["filelogdir"] = test_data_path / "log"
+    prms.Paths.db_filename = "cellpy_db.xlsx"
+    prms.Paths.cellpydatadir = test_data_path / "hdf5"
+    prms.Paths.outdatadir = out_data_path
+    prms.Paths.rawdatadir = test_data_path / "data"
+    prms.Paths.db_path = test_data_path / "db"
+    prms.Paths.filelogdir = test_data_path / "log"
 
     project = "prebens_experiment"
     name = "test"
